///////////////////////////////////////////////////////////////////////////////
// $Revision: 1.22.6.1 $
///////////////////////////////////////////////////////////////////////////////
#pragma option(strict,on)
#include 'slick.sh'



/**
 * Changes the current directory.  The <b>Change Directory dialog box</b> is 
 * displayed to prompt you for the directory to change to.  This command 
 * supports directory aliases and may change the current directory in the 
 * build window.
 * 
 * @return Returns 0 if successful.
 * 
 * @see cd
 * @see cdd
 * @see alias_cd
 *
 * @categories File_Functions
 * 
 */
_command gui_cd() name_info(FILE_ARG " "MORE_ARG','VSARG2_EDITORCTL)
{
   _macro_delete_line();
#if __UNIX__
   _str reinit='-hideondel';
#endif
   typeless result=show('_cd_form -modal -reinit -xy');
   if (result=='') {
      return(COMMAND_CANCELLED_RC);
   }
   _macro('m',_macro('s'));
   _macro_call('cd',result);
   typeless status=cd(result);
   if (status) {
      _message_box(nls("Unable to change directory to '%s'\n",result):+
                   get_message(status));
   }
   return(status);
}

#define ORIGINAL_DIR   _sdname.p_user
#define FINDFILE_NAME  ctlcombo_label.p_user
#define DIR_MUST_EXIST_INDEX 0


defeventtab _cd_form;
void _cd_form.on_create()
{
   _OpenTBDisableRefreshCallback(true);
}

void _cd_form.on_destroy()
{
   _OpenTBDisableRefreshCallback(false);
}

_cd_form.on_resize()
{
   int button_width  = _sdok.p_width;
   int button_height = _sdok.p_height;
   int horz_margin   = ctlcombo_label.p_x;
   int vert_margin   = ctlcombo_label.p_y;

   // force size of dialog to remain reasonable
   if (p_width < button_width*3) {
      p_width = button_width*3;
   }
   if (p_height < button_height*13) {
      p_height = button_height*13;
   }

   int clientHeight = _dy2ly(SM_TWIP, p_active_form.p_client_height);
   int clientWidth = _dx2lx(SM_TWIP, p_active_form.p_client_width);

   int motion_y = clientHeight-vert_margin-_sdsave_settings.p_y-_sdsave_settings.p_height;
   int motion_x = clientWidth-horz_margin-_sdsave_settings.p_x-_sdsave_settings.p_width;

   _sdname.p_width      += motion_x;
   _sdok.p_x            += motion_x;
   ctlcancel.p_x        += motion_x;
   ctlhelp.p_x          += motion_x;
   _sdexpand_alias.p_x  += motion_x;
   _sdprocess_chdir.p_x += motion_x;
   ctlrecursive.p_x     += motion_x;
   ctlDriveLabel.p_y    += motion_y;
   _sddrives.p_y        += motion_y;
   _sddrives.p_width    += motion_x;
   _sdsave_settings.p_x += motion_x;
   _sdsave_settings.p_y += motion_y;
   _sddir_list.p_width  = ctlrecursive.p_x-2*horz_margin;
   _sddir_list.p_height = ctlDriveLabel.p_y - _sddir_list.p_y - 30;
}
_sddir_list.on_resize()
{
   // there are some strange extra calls to this
   // when the user changes the selection, even
   // after fixing the on_update code.  So just
   // try to force it to the correct size.
   p_width=_sddrives.p_width;
   p_height = ctlDriveLabel.p_y - p_y - 30;
}
_sdsave_settings.lbutton_up()
{
   def_cd=0;
   def_cd= def_cd | (_sdprocess_chdir.p_value);
   def_cd= def_cd | ((_sdexpand_alias.p_value)<<1);
   _config_modify_flags(CFGMODIFY_DEFVAR);
   save_config();
   _macro('m',_macro('s'));
   _macro_append('_config_modify_flags(CFGMODIFY_DEFVAR);');
   _macro_append('def_cd='def_cd";");
}

_sdok.lbutton_up()
{
   _str path=_sdname.p_text;
   if (path=='') return('');
   if(_sdexpand_alias.p_value && _sdexpand_alias.p_visible){
      _str multi_line_info="";
      path = get_alias(path,multi_line_info);
      typeless multi_line_flag, file_already_loaded, old_view_id, alias_view_id;
      parse multi_line_info with multi_line_flag file_already_loaded old_view_id alias_view_id .;
      if ( multi_line_flag ) {
         _message_box('Multi-line alias not allowed.');
         p_window_id=_sdname;
         _set_sel(1,length(p_text)+1);_set_focus();
         return(1);
      }

   }
   if (FINDFILE_NAME!='') {
      _str filename=concat_path_and_file(path,FINDFILE_NAME);
      if (!file_exists(filename)) {
         _message_box(nls("The file '%s' does not exist.",filename));
         p_window_id=_sdname;
         _set_sel(1,length(p_text)+1);_set_focus();
         return('');
      }
   } else if ( _GetDialogInfo(DIR_MUST_EXIST_INDEX)==true &&
               !isdirectory(path)) {
      _message_box(nls("The directory '%s' does not exist.",path));
      p_window_id=_sdname;
      _set_sel(1,length(p_text)+1);_set_focus();
      return('');
   }
   typeless result='';
   if( _sdprocess_chdir.p_visible ) {
      result= (_sdprocess_chdir.p_value)?'+p':'-p';
   }
   if( _sdexpand_alias.p_visible ) {
      result= result:+((_sdexpand_alias.p_value)?' +a':' -a');
   }
   if (ctlrecursive.p_visible && ctlrecursive.p_value) {
      result=result:+' +r';
   }
   result=result:+((result:!='')?(' '):('')):+maybe_quote_filename(_sdname.p_text);
   if (_sdname.p_text!='') {
      _append_retrieve(_sdname,_sdname.p_text);
   }
   p_active_form._delete_window(strip(result));
}
/**
 * @categories Forms
 */
_sddir_list.on_create(_str title="",boolean expand_alias_invisible=false,
                      boolean process_chdir_invisible=false,
                      boolean save_settings_invisible=false,
                      boolean ShowRecursive=false,
                      _str find_file="", _str find_path="",
                      boolean path_must_exist=true)
{
#if __UNIX__
   ctlDriveLabel.p_visible = false;
   _sddrives.p_visible = false;
//   _sddir_list.p_height = 2964;
#endif
   _str text=getcwd();
   ORIGINAL_DIR=text;
   _sdname.set_command(text,1,length(text)+1);
   _sdprocess_chdir.p_value=(def_cd & 1);
   // This fixes bug with saving savetings.  Need p_value to be 1 and not 2
   if (def_cd & 2) {
      _sdexpand_alias.p_value=1;
   }
   int list_wid=_sdname.p_cb_list_box;
   list_wid._retrieve_list();
   if( title!='' ) {
      p_active_form.p_caption=title;
   }
   if (find_file!='') {
      ctlcombo_label.p_caption=nls("Enter path for:  %s",find_file);
      _sdname.p_completion=FILE_ARG;
   }
   if( expand_alias_invisible ) {
      _sdexpand_alias.p_visible=0;
   }
   if( process_chdir_invisible ) {
      _sdprocess_chdir.p_visible=0;
   }
   if( save_settings_invisible ) {
      _sdsave_settings.p_visible=0;
   }
   if (ShowRecursive) {
      ctlrecursive.p_visible=1;
   }
   _SetDialogInfo(DIR_MUST_EXIST_INDEX,path_must_exist);
   FINDFILE_NAME=find_file;
   if (find_path!='') {
      _dlpath(find_path, true, def_filelist_show_dotfiles);
   } else {
      _dlpath(getcwd(), true, def_filelist_show_dotfiles);
   }
}
_sddir_list.on_change(int reason)
{
   _str text=_dlpath();
   _sdname.set_command(text,1);
   switch (reason) {
   case CHANGE_PATH:
      int width=_find_longest_line();
      int cwidth=_dx2lx(p_xyscale_mode,p_client_width-p_windent_x);
      cwidth=cwidth-_twips_per_pixel_x()*5;
      //messageNwait('width='width' cwth='cwidth' p_width='p_width);
      if (width>cwidth) {
         int diff_x=width-cwidth;
         p_active_form.p_width=p_active_form.p_width+diff_x;
         call_event(p_active_form,ON_RESIZE);
      }
   }
}

void _sddir_list.'ENTER',lbutton_double_click()
{
   mou_hour_glass(1);
   _str new_path = _dlBuildSelectedPath();
   _dlpath(new_path, 1, def_filelist_show_dotfiles);
   _lbselect_line();
   mou_hour_glass(0);
   // Reset the button press counting
   get_event('B');
}

#if _NAME_HAS_DRIVE
_sddrives.on_change(int reason)
{
   switch (reason) {
   case CHANGE_DRIVE:
      _sddir_list._dlpath(_dvldrive());
   }
}
#endif

_sdname.on_change(int reason)
{
   _str dirname=p_text;
   if (isdirectory(dirname)) {
      // did we have a trailing filesep?
      boolean had_filesep=(last_char(dirname)==FILESEP);
      // update the directory list
      typeless start_pos;
      _get_sel(start_pos);
      _sddir_list._dlpath(dirname, false, def_filelist_show_dotfiles);
      _set_sel(start_pos);
      if (last_char(p_text)==FILESEP && !had_filesep) {
         p_text=substr(p_text,1,length(p_text)-1);
         // Select everything from startpos to the end of the new text,
         // so it is easy for the user to blast it by typing over the
         // selection.
         _set_sel(start_pos,length(p_text)+1);
      }
   }
   // enable/disable the OK button
   _str filename=FINDFILE_NAME;
   if (filename!='') {
      if (last_char(dirname)!=FILESEP) {
         dirname=dirname:+FILESEP;
      }
      _sdok.p_enabled = file_exists(dirname:+filename);
   }
}

int def_symlinks=1;

//#define ORIGINAL_DIR   _sdname.p_user
defeventtab _addtree_form;
_addtree_form.on_resize()
{
   int button_width  = _sdok.p_width;
   int button_height = _sdok.p_height;
   int horz_margin   = ctlcombo_label.p_x;
   int vert_margin   = ctlcombo_label.p_y;

   // force size of dialog to remain reasonable
   if (p_width < button_width*3) {
      p_width = button_width*3;
   }
   if (p_height < button_height*13) {
      p_height = button_height*13;
   }

   int clientHeight = _dy2ly(SM_TWIP, p_active_form.p_client_height);
   int clientWidth = _dx2lx(SM_TWIP, p_active_form.p_client_width);
   int ct_y = (_fileexclude.p_visible) ? (_fileexclude.p_y + _fileexclude.p_height) : (_sddrives.p_y + _sddrives.p_height);
   int motion_y = clientHeight-vert_margin-ct_y;
   int motion_x = clientWidth-horz_margin-_sdok.p_x-_sdok.p_width;

   _sdname.p_width      += motion_x;
   _sdok.p_x            += motion_x;
   ctlcancel.p_x        += motion_x;
   ctlhelp.p_x          += motion_x;
   ctlrecursive.p_x     += motion_x;
   ctlsymlinks.p_x      += motion_x;
   ctlDriveLabel.p_y    += motion_y;

   _sddrives.p_width    += motion_x;
   _sddrives.p_y        += motion_y;

   ctlexclude_label.p_y += motion_y;
   _fileexclude.p_width += motion_x;
   _fileexclude.p_y     += motion_y;

   _sddir_list.p_width  =  ctlrecursive.p_x-2*horz_margin;
   _sddir_list.p_height =  ctlDriveLabel.p_y-_sddir_list.p_y-30;
}
void _sdok.lbutton_up()
{
   if (isdirectory(_sdname.p_text)) {
      _sddir_list._dlpath(_sdname.p_text);
      return;
   }

   _param1._makeempty();
   _str rest=_sdname.p_text;
   _str cwd=_sddir_list._dlpath();
   for (;;) {
      _str filenames="";
      parse rest with filenames ';' rest ;
      if (filenames=='') break;
      for (;;){
         _str bfilename=parse_file(filenames);
         if (bfilename=='') break;
         _str filename=_absolute2(bfilename,cwd);
         //messageNwait('filename='filename);
         // Check if this is a directory specification.
         _param1[_param1._length()]=filename;
         _str new_path=strip_filename(strip(filename,'B','"')/*filename*/,'N');
         _str name_part=strip_filename(strip(filename,'B','"'),'P');
         // Check if path exists
         if (new_path!=''){
            _str path=new_path;
            if(!isdirectory(new_path)) {
               _message_box(new_path"\n"nls("Path does not exist.")"\n":+
                            nls("Please verify that the correct path was given."),
                            p_active_form.p_caption
                         );
               _str text=_sdname.p_text;_sdname.set_command(text,1,length(text)+1);_sdname._set_focus();
               return;
            }
         }
      }
   }
   if (_sdname.p_text!='') {
      _append_retrieve(_sdname,_sdname.p_text);
   }
   if (_fileexclude.p_visible) {
      if (_fileexclude.p_text != '') {
         _append_retrieve(_fileexclude, _fileexclude.p_text);
      }
      _append_retrieve(0, _fileexclude.p_text, "_addtree_form._fileexclude.p_text");
    }
   _param2=ctlrecursive.p_value;
   _param3=ctlsymlinks.p_value;
   _param4=(_fileexclude.p_visible) ? _fileexclude.p_text : "";
   _save_form_response();
   p_active_form._delete_window(1);
}
_sddir_list.on_create(_str title="",
                      //boolean expand_alias_invisible=false,
                      //boolean process_chdir_invisible=false,
                      //boolean save_settings_invisible=false,
                      //boolean recursive_invisible=true,
                      _str filespec="",
                         //This argument is the list of filespecs that go in the textbox
                         //If this argument is specified, the current directory does not change
                      boolean attempt_retrieval=false,
                         // Attempt to do retrieval, but retain given filespec if
                         // nothing is found

                      boolean use_exclude_filespec=false
                      )
{
#if __UNIX__
   ctlDriveLabel.p_visible = false;
   _sddrives.p_visible = false;
   _sddir_list.p_height = 2964;
#endif
   _str text=getcwd();
   ORIGINAL_DIR=text;
   if (attempt_retrieval) _retrieve_prev_form();
   if (_sdname.p_text=='') _sdname.p_text=filespec;
   int list_wid=_sdname.p_cb_list_box;
   list_wid._retrieve_list();
   _str filetypes="";
   for (filetypes=def_file_types;;) {
      _str line=_parse_line(filetypes,',');
      if (line=='') break;
      if (pos('(*.e;',line)) {
         line=stranslate(line,'(*'_macro_ext';','(*.e;');
      }
      parse line with '('line')';
      list_wid._lbtop();
      typeless status=list_wid._lbsearch(line,_fpos_case);
      if (status && line!=ALLFILES_RE) {
         list_wid._lbbottom();
         list_wid._lbadd_item(line);
      }
   }
   list_wid._lbtop();

   if( title!='' ) {
      p_active_form.p_caption=title;
   }
   ctlrecursive.p_value=1;
#if __UNIX__
   // Use retrieval value
   ctlsymlinks.p_value=def_symlinks;
#else
   ctlsymlinks.p_visible=false;
#endif

   ctlexclude_label.p_visible = use_exclude_filespec;
   _fileexclude.p_visible = use_exclude_filespec;
   _dlpath(getcwd(), false, def_filelist_show_dotfiles);
   if (use_exclude_filespec) {
      _fileexclude.p_cb_list_box._retrieve_list();
      _fileexclude.p_text = _retrieve_value("_addtree_form._fileexclude.p_text");
   }
}
ctlrecursive.lbutton_up()
{
   ctlsymlinks.p_enabled=(p_value!=0);
}
_sddir_list.on_destroy()
{
   if (ORIGINAL_DIR!='') {
      chdir(ORIGINAL_DIR,1);
   }
}
_sddir_list.on_change(int reason)
{
   _str text=_dlpath();
   switch (reason) {
   case CHANGE_PATH:
      int width=_find_longest_line();
      int cwidth=_dx2lx(p_xyscale_mode,p_client_width-p_windent_x);
      cwidth=cwidth-_twips_per_pixel_x()*5;
      //messageNwait('width='width' cwth='cwidth' p_width='p_width);
      if (width>cwidth) {
         int diff_x=width-cwidth;
         p_active_form.p_width=p_active_form.p_width+diff_x;
         call_event(p_active_form,ON_RESIZE);
      }
   }
}

#if _NAME_HAS_DRIVE
_sddrives.on_change(int reason)
{
   switch (reason) {
   case CHANGE_DRIVE:
      _sddir_list._dlpath(_dvldrive());
   }
}
#endif
